@@ -8,6 +8,7 @@
 
 #include <Rcpp.h>
 
+#include "gdal_read.h"
 #include "gdal_sf_pkg.h"
 
 using namespace Rcpp;
@@ -50,7 +51,8 @@ CharacterVector CPL_get_metadata(CharacterVector obj, CharacterVector domain_ite
 	GDALDatasetH ds = GDALOpenEx(obj[0], GDAL_OF_RASTER | GDAL_OF_READONLY, NULL, NULL, 
 		create_options(options).data());
 	CharacterVector ret = get_meta_data(ds, domain_item);
-	GDALClose(ds);
+	if (ds != NULL)
+		GDALClose(ds);
 	return ret;
 }
 
@@ -99,38 +101,43 @@ NumericVector CPL_inv_geotransform(NumericVector gt_r) {
 }
 
 // formerly: stars/src/gdal.cpp
-NumericMatrix CPL_read_gdal_data(Rcpp::List meta, GDALDataset *poDataset, NumericVector nodatavalue, 
-		double resample = 1.0) {
-
-	IntegerVector x = meta["cols"];
-	IntegerVector y = meta["rows"];
-	IntegerVector bands = meta["bands"];
-
-	size_t nx = diff(x)[0] + 1, ny = diff(y)[0] + 1, nbands = diff(bands)[0] + 1;
-
-	if (resample < 1.0)
-		warning("resample < 1: trying to oversample"); // #nocov
-	// resample = 1;  --
-	// still need to find out what resample exactly does, 
-	// and how it interacts with and/or should change or changes geotransform
+NumericVector read_gdal_data(GDALDataset *poDataset, 
+			NumericVector nodatavalue, 
+			int nXOff, 
+			int nYOff, 
+			int nXSize, 
+			int nYSize, 
+			int nBufXSize, 
+			int nBufYSize, 
+			IntegerVector bands,
+			GDALRasterIOExtraArg *resample
+		) {
 
 	// collapse x & y into rows, redim later:
-	NumericMatrix mat( floor(nx / resample) * floor(ny / resample), nbands );
+	NumericVector vec( 1.0 * nBufXSize * nBufYSize * bands.size() ); 
+	// floor returns double -> no integer overflow
 
-	IntegerVector band_index(nbands);
-	for (int j = 0, i = bands(0); i <= bands(1); i++)
-		band_index(j++) = i;
-	// read the full set of bands:
-	CPLErr err = poDataset->RasterIO( GF_Read,
-			x(0) - 1, y(0) - 1,
-			nx, ny,
-			mat.begin(),
-			floor(nx / resample), floor(ny / resample),
-			GDT_Float64, nbands, band_index.begin(), 0, 0, 0);
-	if (err == CE_Failure)
+	// read bands:
+	if (poDataset->RasterIO( GF_Read,
+			nXOff,
+			nYOff,
+			nXSize, 
+			nYSize,
+			vec.begin(),
+			nBufXSize,
+			nBufYSize,
+			GDT_Float64, 
+			bands.size(),
+			bands.begin(), 
+			0, 
+			0, 
+			0, 
+			resample) == CE_Failure)
 		stop("read failure"); // #nocov
 
-	for (int band = bands(0); band <= bands(1); band++) { // unlike x & y, band is 1-based
+	// scale && set NA:
+	for (int i = 0; i < bands.size(); i++) {
+		int band = bands(i);
 		GDALRasterBand *poBand = poDataset->GetRasterBand( band );
 		// NumericVector nodatavalue = NumericVector::create(NA_REAL);
 		// int success = 0, 
@@ -147,40 +154,46 @@ NumericMatrix CPL_read_gdal_data(Rcpp::List meta, GDALDataset *poDataset, Numeri
 			offset = poBand->GetOffset(NULL);
 		// char *units = poBand->GetUnits();
 		if (! NumericVector::is_na(nodatavalue[0]) || has_offset || has_scale) {
-			NumericVector nv = mat(_, band - 1);
-			for (int i = 0; i < nv.size(); i++) {
-				if (nv[i] == nodatavalue[0])
-					nv[i] = NA_REAL; // #nocov
+			for (R_xlen_t i = 0; i < Rf_xlength(vec); i++) {
+				if (vec[i] == nodatavalue[0])
+					vec[i] = NA_REAL; // #nocov
 				else
-					nv[i] = (nv[i] * scale) + offset;
+					vec[i] = (vec[i] * scale) + offset;
 			}
-			mat(_, band - 1) = nv;
 		}
 		checkUserInterrupt();
 	}
 
-	// dim:
+	// set dim attr:
 	IntegerVector dims;
-	if (diff(bands)[0] == 0) { // one band:
-		dims = IntegerVector::create(nx / resample, ny / resample);
+	if (bands.size() == 1) { // single band:
+		dims = IntegerVector::create(nBufXSize, nBufYSize);
 		dims.attr("names") = CharacterVector::create("x", "y");
-	} else { // redim:
-		dims = IntegerVector::create(nx / resample, ny / resample, diff(bands)[0] + 1); // #nocov
+	} else { // multiple bands:
+		dims = IntegerVector::create(nBufXSize, nBufYSize, bands.size()); // #nocov
 		dims.attr("names") = CharacterVector::create("x", "y", "band"); // #nocov
 	}
-	mat.attr("dim") = dims;
-	return mat;
+	vec.attr("dim") = dims;
+	return vec;
+}
+
+int get_from_list(List lst, const char *name, int otherwise) {
+	if (lst.containsElementNamed(name)) {
+		IntegerVector ret = lst[name]; // #nocov
+		return(ret[0]);                // #nocov
+	} else
+		return(otherwise);
 }
 
 // [[Rcpp::export]]
 List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVector driver,
-		bool read_data, NumericVector NA_value, double resample = 1.0) {
+		bool read_data, NumericVector NA_value, List RasterIO_parameters) {
 // reads and returns data set metadata, and if read_data is true, adds data array
-    GDALDataset  *poDataset = (GDALDataset *) GDALOpenEx(fname[0], GA_ReadOnly,
+    GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(fname[0], GA_ReadOnly,
 		driver.size() ? create_options(driver).data() : NULL,
 		options.size() ? create_options(options).data() : NULL,
 		NULL);
-    if( poDataset == NULL ) {
+    if (poDataset == NULL) {
 		Rcout << "trying to read file: " << fname[0] << std::endl; // #nocov
         stop("file not found"); // #nocov
 	}
@@ -191,7 +204,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 
 	/*
 	if (poDataset->GetRasterCount() == 0)
-		stop("zero bands");
+		stop("zero bands"); -->> this indicates there are (only) subdatasets
 	*/
 
 	// geotransform:
@@ -204,6 +217,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		err == CE_None ? adfGeoTransform[3] : NA_REAL,
 		err == CE_None ? adfGeoTransform[4] : NA_REAL,
 		err == CE_None ? adfGeoTransform[5] : NA_REAL);
+	bool geo_transform_set = (err == CE_None);
 
 	// CRS, projection:
 	const char *wkt = poDataset->GetProjectionRef();
@@ -214,7 +228,11 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		// proj4string:
 		OGRSpatialReference *sr = new OGRSpatialReference;
 		char **ppt = (char **) &wkt;
+#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
 		sr->importFromWkt(ppt);
+#else
+		sr->importFromWkt( (const char**) ppt);
+#endif
 		char *proj4; 
 		if (sr->exportToProj4(&proj4) != OGRERR_NONE) // need to error check?
 			stop("failure to export SRS to proj.4"); // #nocov
@@ -245,12 +263,55 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 			sub = get_meta_data(poDataset, "SUBDATASETS"); // #nocov
 	}
 
+	// image dimension:
+	int nXOff = get_from_list(RasterIO_parameters, "nXOff", 1) - 1;
+	int nYOff = get_from_list(RasterIO_parameters, "nYOff", 1) - 1;
+	int nXSize = get_from_list(RasterIO_parameters, "nXSize", poDataset->GetRasterXSize() - nXOff);
+	int nYSize = get_from_list(RasterIO_parameters, "nYSize", poDataset->GetRasterYSize() - nYOff);
+	int nBufXSize = get_from_list(RasterIO_parameters, "nBufXSize", nXSize);
+	int nBufYSize = get_from_list(RasterIO_parameters, "nBufYSize", nYSize);
+
+	// bands:
+	IntegerVector bands; 
+	if (RasterIO_parameters.containsElementNamed("bands"))
+		bands = RasterIO_parameters["bands"]; // #nocov
+	else {
+		bands = IntegerVector(poDataset->GetRasterCount());
+		for (int j = 0; j < bands.size(); j++)
+			bands(j) = j + 1; // bands is 1-based
+	}
+
+	// resampling method:
+	GDALRasterIOExtraArg resample;
+	INIT_RASTERIO_EXTRA_ARG(resample);
+
+	if (RasterIO_parameters.containsElementNamed("resample")) { // #nocov start
+		CharacterVector res = RasterIO_parameters["resample"];
+		if (res[0] == "bilinear")
+			resample.eResampleAlg = GRIORA_Bilinear;
+		else if (res[0] == "cubic")
+			resample.eResampleAlg = GRIORA_Cubic;
+		else if (res[0] == "cubic_spline")
+			resample.eResampleAlg = GRIORA_CubicSpline;
+		else if (res[0] == "lanczos")
+			resample.eResampleAlg = GRIORA_Lanczos;
+		else if (res[0] == "average")
+			resample.eResampleAlg = GRIORA_Average;
+		else if (res[0] == "mode")
+			resample.eResampleAlg = GRIORA_Mode;
+		else if (res[0] == "Gauss")
+			resample.eResampleAlg = GRIORA_Gauss;
+		else if (res[0] == "nearest_neighbour")
+			resample.eResampleAlg = GRIORA_NearestNeighbour;
+		else stop("unknown method for resample"); // #nocov end
+	} 
+
 	List ReturnList = List::create(
 		_["filename"] = fname,
 		_["driver"] = Driver,
-		_["cols"] = NumericVector::create(1, poDataset->GetRasterXSize()),
-		_["rows"] = NumericVector::create(1, poDataset->GetRasterYSize()),
-		_["bands"] = NumericVector::create(1, poDataset->GetRasterCount()),
+		_["cols"] = NumericVector::create(nXOff + 1, nXOff + nBufXSize),
+		_["rows"] = NumericVector::create(nYOff + 1, nYOff + nBufYSize),
+		_["bands"] = bands,
 		_["proj_wkt"] = proj,
 		_["proj4string"] = p4,
 		_["geotransform"] = geotransform,
@@ -261,10 +322,27 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		_["meta"] = get_meta_data(poDataset, CharacterVector::create()),
 		_["band_meta"] = get_band_meta_data(poDataset)
 	);
-	if (read_data)
-		ReturnList.attr("data") = CPL_read_gdal_data(ReturnList, poDataset, nodatavalue, resample);
-
+	if (read_data) {
+		ReturnList.attr("data") = read_gdal_data(poDataset, nodatavalue, nXOff, nYOff, 
+			nXSize, nYSize, nBufXSize, nBufYSize, bands, &resample);
+	}
 	GDALClose(poDataset);
+
+	// adjust geotransform & offset if Buf?Size was set:
+	if ((nXSize != nBufXSize || nYSize != nBufYSize) && geo_transform_set) { // #nocov start
+		if (geotransform[2] != 0.0 || geotransform[4] != 0.0)
+			stop("reading affine grids with resampling would result in a wrong geotransform; please file an issue"); // #nocov
+		double ratio_x = (1.0 * nXSize) / nBufXSize;
+		double ratio_y = (1.0 * nYSize) / nBufYSize;
+		geotransform[1] = geotransform[1] * ratio_x;
+		geotransform[5] = geotransform[5] * ratio_y;
+		ReturnList["geotransform"] = geotransform;
+		nXOff = (int) (nXOff / ratio_x);
+		nYOff = (int) (nYOff / ratio_y);
+		ReturnList["cols"] = NumericVector::create(nXOff + 1, nXOff + nBufXSize);
+		ReturnList["rows"] = NumericVector::create(nYOff + 1, nYOff + nBufYSize);
+	} // #nocov end
+
 	return ReturnList;
 }
 
@@ -322,7 +400,7 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 	for (int i = 0; i < gt.length(); i++)
 		adfGeoTransform[i] = gt[i];
 	if (poDstDS->SetGeoTransform( adfGeoTransform ) != CE_None)
-		stop("SetGeoTransform: error"); // #nocov
+		warning("SetGeoTransform() returned an error: not available?"); // #nocov
 
 	// CRS:
 	if (p4s.length() != 1)
@@ -358,7 +436,7 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 	}
 
 	// write the whole lot:
-	if (poDstDS->RasterIO( GF_Write, 0, 0, dims[0], dims[1],
+	if (poDstDS->RasterIO(GF_Write, 0, 0, dims[0], dims[1],
 			x.begin(), dims[0], dims[1], GDT_Float64, dims[2], NULL, 0, 0, 0, NULL) == CE_Failure)
 		stop("read failure"); // #nocov
 
